/**
 *
 * @author Matias Contreras, Stefano Aguilera
 */

package sensado_remoto;

import java.net.*;
import java.io.*;
import java.util.*;

import java.util.logging.Level;
import java.util.logging.Logger;
import javafx.application.Application;
import javafx.application.Platform;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.scene.Scene;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.layout.HBox;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.stage.Stage;
import jssc.SerialPort;
import static jssc.SerialPort.MASK_RXCHAR;
import jssc.SerialPortEvent;
import jssc.SerialPortEventListener;
import jssc.SerialPortException;
import jssc.SerialPortList;

public class Sensado_Remoto extends Application{
    static ArrayList<String> datos = new ArrayList<String>();
    
    SerialPort arduinoPort = null;
    ObservableList<String> portList;
    Label labelValue, text;
     
    private void detectPort(){
        portList = FXCollections.observableArrayList();
        String[] serialPortNames = SerialPortList.getPortNames();
        for(String name: serialPortNames){
            System.out.println("Conexión Java - Arduino");
            System.out.println("Puerto: " + name);
            portList.add(name);
        }
    }
    
    @Override
    public void start(Stage primaryStage){
        text = new Label();
        text.setFont(new Font("Arial", 39));
        text.setText("Level: ");
        labelValue = new Label();
        labelValue.setFont(new Font("Arial", 40));
        
        detectPort();
        final ComboBox comboBoxPorts = new ComboBox(portList);
        comboBoxPorts.valueProperty().addListener(new ChangeListener<String>(){
            @Override
            public void changed(ObservableValue<? extends String> observable, 
                    String oldValue, String newValue){
                System.out.println(newValue);
                disconnectArduino();
                connectArduino(newValue);
            }
        });
        
        VBox vBox = new VBox();
        HBox hBox = new HBox();
        hBox.getChildren().addAll(new Text("   "), text, new Text("   "), labelValue);
        vBox.getChildren().addAll(comboBoxPorts, new Text(" "), hBox);
        
        StackPane root = new StackPane();
        root.getChildren().add(vBox);
   
        Scene scene = new Scene(root, 250, 150);
        primaryStage.setTitle("ELO330 - Sensado Remoto de Recipientes Hídricos");
        primaryStage.setScene(scene);
        primaryStage.show();
    }

    public boolean connectArduino(String port){
        System.out.println("ConnectArduino(String)");
        boolean success = false;
        SerialPort serialPort = new SerialPort(port);
        try{
            serialPort.openPort();
            serialPort.setParams(
                    SerialPort.BAUDRATE_9600,
                    SerialPort.DATABITS_8,
                    SerialPort.STOPBITS_1,
                    SerialPort.PARITY_NONE);
            serialPort.setEventsMask(MASK_RXCHAR);
            serialPort.addEventListener(new SerialPortEventListener() {
                @Override
                public void serialEvent(SerialPortEvent serialPortEvent) {
                    if(serialPortEvent.isRXCHAR()){
                        try{
                            byte[] b = serialPort.readBytes();
                            int value = b[0] & 0xff;            /* convert to int */
                            String st = String.valueOf(value);
                            datos.add(st);
                            
                            /* Update label in ui thread */
                            Platform.runLater(() -> { labelValue.setText(st); });
                            
                        }catch(SerialPortException ex){
                            Logger.getLogger(Sensado_Remoto.class.getName())
                                    .log(Level.SEVERE, null, ex);
                        }
                    }
                }
            });
            
            arduinoPort = serialPort;
            success = true;
        } catch(SerialPortException ex){
            Logger.getLogger(Sensado_Remoto.class.getName())
                    .log(Level.SEVERE, null, ex);
            System.out.println("SerialPortException: " + ex.toString());
        }
        return success;
    }
    
    public void disconnectArduino(){
        System.out.println("DisconnectArduino()");
        if(arduinoPort != null){
            try{
                arduinoPort.removeEventListener();
                if(arduinoPort.isOpened()){
                    arduinoPort.closePort();
                }
                
            }catch(SerialPortException ex){
                Logger.getLogger(Sensado_Remoto.class.getName())
                        .log(Level.SEVERE, null, ex);
            }
        }
    }

    @Override
    public void stop() throws Exception{
        disconnectArduino();
        super.stop();
    }
    
    static class ClientDataSensor{
        private Socket socket;
        private BufferedReader in;
        private PrintWriter out;
        private Sensor sensor;

        public ClientDataSensor(InetAddress addr, int PORT, Sensor data){
          sensor = data;

          try{
            socket = new Socket(addr, PORT);
          }catch(IOException e){
            System.out.println("(Error: Socket creation failed.)");
            /* if the creation of the socket fails, nothing needs to be cleaned up. */

          }try{    
            in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
            out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())), true);

            /* send data to foro_setup */
            out.println("U" + "," + sensor.name + "," + sensor.location + "," + sensor.dataSensor);

            /* receive data from foro_setup */
            System.out.println("\nResumen de Datos del Sensor");
            System.out.println("Nombre: " + sensor.name);
            System.out.println("Ubicacion: " + sensor.location);

            /* command */
            String str = in.readLine();
            if(str != null){
              char command = str.charAt(0);
              if(command == 'O')
                System.out.println("Ingresados correctamente. Flujo de datos de " + sensor.name  + " recibidos y enviados al servidor.\n");
              if(command == 'F')
                System.out.println("El sensor " + sensor.name + " no pudo ser ingresado. Hable con el creador del proyecto.\n");
            }

          }catch(IOException e){
            /* the socket should be closed on any failures other than the socket constructor */
            try{
              socket.close();
            }catch(IOException e2){}
          }
          /* otherwise the socket will be closed bythe run() method of the thread */
        }
    }
    
    public static void main(String[] args) throws IOException, InterruptedException{
        /* program syntax */
        if(args.length != 2){
          System.out.println("Usage: java sensado_data <forum_server_machine> <server_attention_port>");
          System.exit(0);
        }

        /* sensor title for data submission */
        System.out.println("\nSensado Remoto de Recipientes Hídricos Artesanales (Data Sender)");

        /* sensor data request */
        BufferedReader buffer = new BufferedReader(new InputStreamReader(System.in));
        System.out.print("Ingrese Nombre del Sensor: ");
        String nombre = buffer.readLine();
        System.out.print("Ingrese Ubicación del Sensor: ");
        String ubicacion = buffer.readLine();

        /* Ingresar datos desde Arduino */
        System.out.println("Captando Datos desde Sensor...");
        
        /* launch graphic application */
        launch(args);
        String points = null;
        for(int i = 0; i < datos.size(); i++){
            if(i == 0) points = datos.get(i) + "-";
            else if(i == (datos.size()-1)) points += datos.get(i);
            else points += datos.get(i) + "-";
        }
        
        /* create sensor */
        Sensor sensor = new Sensor(nombre, ubicacion, points);
        System.out.println("(Waiting for server response...)");

        /* variables */
        String server = args[0];                  /* machine server */
        int PORT = Integer.parseInt(args[1]);     /* port */

        /* call client */
        InetAddress addr = InetAddress.getByName(server);
        new ClientDataSensor(addr, PORT, sensor);
        datos.clear();
    }
}
