/**
 * Programa para interpretar los datos recibidos desde el sensor.
 * Autores: Stefano Aguilera, Matias Contreras
 */

import java.net.*;
import java.io.*;
import java.lang.Runtime;

class ClientSensorGraphics{
  private String letter;
  private Socket socket;
  private BufferedReader in;
  private PrintWriter out;
  
  public ClientSensorGraphics(InetAddress addr, int PORT, String clean){
    letter = clean;
    
    try{
      socket = new Socket(addr, PORT);
    }catch(IOException e){
      System.out.println("(Error: Socket creation failed.)");
      /* if the creation of the socket fails, nothing needs to be cleaned up. */

    }try{
      in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
      out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())), true);

      if(letter.equals("n")){
        /* send command to foro_setup */
        out.println("E");
        String str = in.readLine();

        if(str != null){
          char command = str.charAt(0);

          if(command == 'O'){
            /* split data */
            String[] parts = str.split(",");
            String nombre = parts[1];
            String ubicacion = parts[2];
            String[] distancia = parts[3].split("-");

            System.out.println("\nSensor Seleccionado.");
            System.out.println("Nombre del Sensor: " + nombre);
            System.out.println("Ubicación del Sensor: " + ubicacion);
            System.out.println("Inicia el tratamiento de datos desde el sensor " + nombre + "...\n");

    			  File verifyLevelFile = new File("Level" + nombre + ".txt");
    		    if(verifyLevelFile.exists()){
    		        boolean abc = verifyLevelFile.delete();
    		        if(!abc) System.out.println("Error: Something is wrong.");
    		    }

    		    FileWriter nameLevelFile = new FileWriter("Level" + nombre + ".txt");
    		    PrintWriter levelFile = new PrintWriter(nameLevelFile);
    		    for(int j = 0; j < distancia.length; j++){
                levelFile.print(j+1);
    		        levelFile.println(" " + distancia[j]);
            }

    		    levelFile.close();

      			File verifyGnuFile = new File("commandFor" + nombre + ".txt");
            if(verifyGnuFile.exists()){
              boolean abc = verifyGnuFile.delete();
              if(!abc) System.out.println("Error: Something is wrong.");
            }

      			FileWriter nameGnuFile = new FileWriter("commandFor" + nombre + ".txt");
      			PrintWriter gnuFile = new PrintWriter(nameGnuFile);
      			gnuFile.println("set terminal png");
      			gnuFile.println("set output \"Level" + nombre + ".png\"");
      			gnuFile.println("set title \"Sensado Remoto de Recipientes Hídricos\"");
      			gnuFile.println("set xlabel \"Time [s]\"");
      			gnuFile.println("set ylabel \"Level [cm]\"");
      			gnuFile.println("set key right bottom");
      			gnuFile.println("plot \"Level" + nombre + ".txt\" with linespoints");
      			gnuFile.close();

      			Process gnuCommand = Runtime.getRuntime().exec("gnuplot commandFor" + nombre + ".txt");
            try{
              gnuCommand.waitFor();   /* Wait for gnuCommand */
            }catch(Exception e){}

            File deleteLevelData = new File("Level" + nombre + ".txt");
            deleteLevelData.delete();
            File deleteCommandsFile = new File("commandFor" + nombre + ".txt");
            deleteCommandsFile.delete();
          }

          else if(command == 'E'){
            System.out.println("No se pueden obtener datos dado que no se ha conectado ningún sensor.\n");
          }
        }
      }

      else if(letter.equals("s")){
        /* send command to foro_setup */
        System.out.println("\nSolicitud para Concluir la Recepción de Datos desde el Sensor.");
        out.println("S");
        System.out.println("(OK: Closing server...)");
        System.out.println("Memoria liberada, servidor cerrado.\n");
      }

    }catch(IOException e){
      /* the socket should be closed on any failures other than the socket constructor */
      try{
        socket.close();
      }catch(IOException e2){}
    }
    /* otherwise the socket will be closed bythe run() method of the thread */
  }
}

public class sensado_graphics{
  public static void main(String[] args) throws IOException, InterruptedException{
    /* program syntax */
    if((args.length == 2) != (args.length != 3)){
      System.out.println("Usage: java sensado_graphics <forum_server_machine> <server_attention_port> (s)");
      System.exit(0);
    }

    /* variables */
    String server = args[0];                  /* forum machine server */
    int PORT = Integer.parseInt(args[1]);     /* port */
    String command = "s";                     /* clean data */
    
    InetAddress addr = InetAddress.getByName(server);

    /* sensor title for data graphics */
    System.out.println("\nSensado Remoto de Recipientes Hídricos Artesanales (Create Graphics)");
    System.out.println("(Waiting for server response...)");

    if(args.length == 2){
      new ClientSensorGraphics(addr, PORT, "n");
    }

    else if((args.length == 3) && command.equals(args[2])){
      new ClientSensorGraphics(addr, PORT, "s");
    }
  }
}