/**
 * Programa para configurar el inicio y termino de sensado remoto de recipientes.
 * Autores: Stefano Aguilera, Matias Contreras
 */

import java.io.*;
import java.net.*;

class ServerSensado extends Thread{
  private Socket socket;
  private BufferedReader in;
  private PrintWriter out;
  private SensorData sensorData;
  private Sensor sensor, sensado;
  
  public ServerSensado(Socket s, SensorData sD) throws IOException{
    socket = s;
    sensorData = sD;

    /* enable auto-flush */
    in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
    out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())), true);
    /* If any of the above calls throw an exception, the caller is responsible 
     * for closing the socket. Otherwise the thread will close it. */

    start();    /* calls run() */
  }
  
  public void run(){
    try{
      while(true){
		String str = in.readLine();
        char command = '\0';
		if(str != null) command = str.charAt(0);

        /* for sensado_data */
        if(command == 'U'){
          /* split data */
          String[] parts = str.split(",");
          String nombre = parts[1];
          String ubicacion = parts[2];
          String distancia = parts[3];
          sensor = new Sensor(nombre, ubicacion, distancia);

          /* print data in console */
          System.out.println("\nResumen de Datos del Sensor");
          System.out.println("Nombre: " + nombre);
          System.out.println("Ubicación: " + ubicacion);

          if(sensorData.addSensor(sensor)){
          	out.println("O");
            System.out.println("\nInformación y datos del sensor agregados, avisado al agricultor/granjero.");
          	System.out.println("(OK: Name and location added correctly.)");

          }else{
            out.println("F");
            System.out.println("\nNo se aceptan mas sensores. Se ha alcanzado el limite.");
            System.out.println("(Error: Full array. Can't added question.)");
          }
      	}

        /* for sensado_graphics */
        else if(command == 'E'){
          sensado = sensorData.getRandomSensor();
          if(sensado != null){
            System.out.println("Nombre del Sensor: " + sensado.name);
            System.out.println("Ubicación del Sensor: " + sensado.location);

            out.println("O" + "," + sensado.name + "," + sensado.location + "," + sensado.dataSensor);
            System.out.println("\nInformación y datos enviados, recibidos por el moderador.");
            System.out.println("(OK: sended correctly.)");

          }else{
            out.println("E");
            System.out.println("\nNo se han agregado sensores ni información de ellos.");
            System.out.println("(Error: Empty array. Can't send a data of sensors.)");
          }
        }

        /* for close forum */
        else if(command == 'S'){
          System.out.println("\nSolicitud para Concluir la Recepción de Datos desde el Sensor.");
          System.out.println("(OK: Closing server...)");
          sensorData.clearData();
          socket.close();

          System.out.println("Memoria liberada, servidor cerrado.\n");
          System.exit(0);
        }
      }
    }catch (IOException e){
    }finally{
      try{
        socket.close();
      }catch(IOException e){}
    }
  }
}

public class sensado_server{
  public static void main(String[] args) throws IOException{
    /* program syntax */
    if(args.length != 2){
      System.out.println("Usage: java sensado_server <N> <local_listening_port>");
      System.exit(0);
    }

    /* variables */
    int N = Integer.parseInt(args[0]);       /* N */
    int PORT = Integer.parseInt(args[1]);    /* port */

    /* create structures */
    SensorData sensorData = new SensorData(0, N);

    /* create socket */
    ServerSocket s = new ServerSocket(PORT);
	    
    /* project title */
    System.out.println("\nSensado Remoto de Recipientes Hídricos Artesanales (Server)");
    System.out.println("(Waiting for sensors...)");

    try{
      while(true){
        /* server blocks until a connection occurs */
        Socket socket = s.accept();
        try{
          /* calls class to display connection information */
          new ServerSensado(socket, sensorData);

        }catch(IOException e){
          /* if it fails, close the sockets, otherwise the thread will close it */
          socket.close();
        }
      }
    }finally{
        s.close();
    }
  } 
}