/* Constantes */
const int EchoPin = 5;            // Reserva pin D5 para pin echo del sensor
const int TriggerPin = 6;         // Reserva pin D6 para pin trigger del sensor
const int LedPinVerde = 12;       // Reserva pin D12 para led verde
const int LedPinAmarillo = 11;    // Reserva pin D11 para led amarillo
const int LedPinRojo = 10;        // Reserva pin D10 para led rojo
const int high = 100;             // Alto del recipiente de agua
  
/* Variables */
float distancia;        
long tiempo;
 
void setup(){
    Serial.begin(9600);                   // 9600 baudios: comunicación serial
    pinMode(LedPinVerde, OUTPUT);         // Salida: led verde
    pinMode(LedPinAmarillo, OUTPUT);      // Salida: led amarillo
    pinMode(LedPinRojo, OUTPUT);          // Salida: led rojo
    pinMode(TriggerPin, OUTPUT);          // Salida: pin trigger (umbral) del sensor
    pinMode(EchoPin, INPUT);              // Entrada: pin echo (capta datos de distancia) del sensor
}
 
void loop(){
    digitalWrite(TriggerPin, HIGH);     // Se envía un pulso para activar el sensor
    delayMicroseconds(10);              // Delay asociado al pulso
    digitalWrite(TriggerPin, LOW);      // Se apaga el pulso enviado

    // Se mide el pulso de respuesta
    tiempo = pulseIn(EchoPin, HIGH)/2;    // Se divide en 2 por que es el tiempo que el sonido tarda en ir y volver
 
    // Ahora, se calcula la distancia en [cm]:
    // Se sabe que la distancia es igual a la velocidad por el tiempo
    // y que la velocidad del sonido en el aire es de 343 [m/s], sumado
    // a que el tiempo se tiene en millonésimas de segundo
    distancia = float(tiempo * 0.0343);
    
    Serial.write(100 - (int)distancia);         // Escribe bytes en el puerto USB

    if(distancia >= 30){                        // Parámetro de mayor lejanía: mayor o igual a 30[cm]
        digitalWrite(LedPinVerde , HIGH);       // Enciende luz verde
        digitalWrite(LedPinAmarillo , LOW);     // Apaga luz amarilla
        digitalWrite(LedPinRojo , LOW);         // Apaga luz roja
    }
    else if((distancia <= 29) && (distancia >= 15)){      // Distancia de mediana lejanía: 29 - 15[cm]
        digitalWrite(LedPinVerde , LOW);                  // Apaga luz verde
        digitalWrite(LedPinAmarillo , HIGH);              // Enciende luz amarilla
        digitalWrite(LedPinRojo , LOW);                   // Apaga luz roja
    }
    else if (distancia <= 14){                  // Parámetro de menor lejanía: menor o igual a 14[cm]
        digitalWrite(LedPinVerde , LOW);        // Apaga luz verde
        digitalWrite(LedPinAmarillo , LOW);     // Apaga luz amarilla
        digitalWrite(LedPinRojo , HIGH);        // Enciende luz roja
    } 

    delay(1000);    // Delay para imprimir valores en pantalla a un ritmo apreciable
}
